@@ -10,6 +10,8 @@
 from __future__ import annotations
 
 import os
+import socket
+import time
 import uuid
 import webbrowser
 from dataclasses import dataclass
@@ -19,17 +21,18 @@ from typing import Dict, Optional
 from urllib.parse import urlencode
 
 import requests
-from pants.core.util_rules.pants_environment import PantsEnvironment
 from pants.engine.console import Console
+from pants.engine.environment import Environment, EnvironmentRequest
 from pants.engine.fs import CreateDigest, Digest, FileContent, Workspace
 from pants.engine.goal import Goal, GoalSubsystem
 from pants.engine.rules import Get, collect_rules, goal_rule, rule
 from pants.option.global_options import GlobalOptions
 from pants.option.subsystem import Subsystem
 
-from toolchain.pants.auth.client import ACQUIRE_TOKEN_GOAL_NAME, AuthClient, AuthError, AuthToken
-from toolchain.pants.auth.server import AuthFlowHttpServer
+from toolchain.pants.auth.client import ACQUIRE_TOKEN_GOAL_NAME, AuthClient, AuthError
+from toolchain.pants.auth.server import AuthFlowHttpServer, TestPage
 from toolchain.pants.auth.store import AuthStore
+from toolchain.pants.auth.token import AuthToken
 from toolchain.pants.common.network import get_common_request_headers
 from toolchain.pants.common.toolchain_setup import ToolchainSetup
 
@@ -60,6 +63,14 @@ class AccessTokenAcquisitionGoalOptions(GoalSubsystem):
             help="Output method for access token. Outputing the console is useful if the token needs to be provided to CI",
         )
         register("--headless", type=bool, default=False, help="Don't open & use a browser to acquire access token")
+        register(
+            "--test-page",
+            type=TestPage,
+            default=TestPage.NA,
+            advanced=True,
+            help="Helper to test success and error pages w/o triggering auth flow",
+        )
+        register("--description", type=str, default=None, help="Token description")
 
 
 class AuthStoreOptions(Subsystem):
@@ -74,13 +85,19 @@ class AuthStoreOptions(Subsystem):
             help="Relative path (relative to the build root) for where to store and read the auth token",
         )
         register("--from-env-var", type=str, default=None, help="Loads the access token from an environment variable")
-        register("--base-url", type=str, help="auth app base url", default="https://app.toolchain.com/api/v1")
         register(
             "--ci-env-variables",
             type=list,
             help="Environment variables in CI used to identify build (for restricted tokens)",
         )
         register("--org", type=str, default=None, help="organization slug for public repo PRs")
+        register(
+            "--restricted-token-matches",
+            type=dict,
+            default={},
+            advanced=True,
+            help="A dict containing environment variables with their expected values (regex) which need to match in order for the plugin to request a restricted access token.",
+        )
 
 
 class AccessTokenAcquisition(Goal):
@@ -94,6 +111,8 @@ class AccessTokenAcquisitionOptions:
     repo_name: str
     local_port: Optional[int]
     headless: bool
+    test_page: TestPage
+    description: str
 
     @classmethod
     def from_options(
@@ -103,13 +122,15 @@ class AccessTokenAcquisitionOptions:
         store_options: AuthStoreOptions,
         pants_bin_name: str,
         repo_name: str,
+        base_url: str,
     ) -> AccessTokenAcquisitionOptions:
         acquire_values = acquire_options.options
         store_values = store_options.options
         auth_opts = AuthClient.create(
             pants_bin_name=pants_bin_name,
-            base_url=store_values.base_url,
+            base_url=f"{base_url}/api/v1",
             auth_file=store_values.auth_file,
+            context="auth-acquire",
         )
         return cls(
             local_port=acquire_values.local_port,
@@ -117,6 +138,8 @@ class AccessTokenAcquisitionOptions:
             auth_options=auth_opts,
             output=acquire_values.output,
             headless=acquire_values.headless,
+            test_page=acquire_values.test_page,
+            description=acquire_values.description,
         )
 
     @property
@@ -147,17 +170,20 @@ class AccessTokenAcquisitionOptions:
 
 
 @rule
-def construct_auth_store(
+async def construct_auth_store(
     auth_store_config: AuthStoreOptions,
     global_options: GlobalOptions,
     toolchain_setup: ToolchainSetup,
-    pants_environment: PantsEnvironment,
 ) -> AuthStore:
+    options = auth_store_config.options
+    environment = await Get(Environment, EnvironmentRequest(AuthStore.relevant_env_vars(options)))
     return AuthStore(
-        options=auth_store_config.options,
+        context="rules",
+        options=options,
         pants_bin_name=global_options.options.pants_bin_name,
-        env=dict(pants_environment.env),
+        env=dict(environment),
         repo=toolchain_setup.safe_get_repo_name(),
+        base_url=toolchain_setup.base_url,
     )
 
 
@@ -176,7 +202,11 @@ async def acquire_access_token(
         acquire_options=acquire_goal_options,
         store_options=store_options,
         repo_name=repo_name,
+        base_url=toolchain_setup.base_url,
     )
+    if acquire_options.test_page != TestPage.NA:
+        _test_local_server(acquire_options)
+        return AccessTokenAcquisition(exit_code=0)
     try:
         auth_token = _acquire_token(console, acquire_options)
     except AuthError as error:
@@ -201,6 +231,19 @@ def _acquire_token(console: Console, options: AccessTokenAcquisitionOptions) ->
     return _acquire_token_with_browser(console, options)
 
 
+def _test_local_server(options: AccessTokenAcquisitionOptions):
+    with AuthFlowHttpServer.create_server(port=options.local_port, expected_state=str(uuid.uuid4())) as http_server:
+        http_server.start_thread()
+        server_url = http_server.get_test_url(options.test_page)
+        success = webbrowser.open(server_url, new=1, autoraise=True)
+        if not success:
+            http_server.shutdown()
+            raise AuthError(
+                f"Failed to open web browser. {ACQUIRE_TOKEN_GOAL_NAME} can't continue.", context="auth-acquire"
+            )
+        time.sleep(4)  # sleep to allow the browser to load the page from the server.
+
+
 def _acquire_token_with_browser(console: Console, options: AccessTokenAcquisitionOptions) -> AuthToken:
     state = str(uuid.uuid4())
     with AuthFlowHttpServer.create_server(port=options.local_port, expected_state=state) as http_server:
@@ -211,21 +254,26 @@ def _acquire_token_with_browser(console: Console, options: AccessTokenAcquisitio
         success = webbrowser.open(options.get_auth_url(options.repo_name, params), new=1, autoraise=True)
         if not success:
             http_server.shutdown()
-            raise AuthError(f"Failed to open web browser. {ACQUIRE_TOKEN_GOAL_NAME} can't continue.")
+            raise AuthError(
+                f"Failed to open web browser. {ACQUIRE_TOKEN_GOAL_NAME} can't continue.", context="auth-acquire"
+            )
         token_code = http_server.wait_for_code()
-        return _exchage_code_for_token(console, options, token_code)
+        desc = _get_token_desc(options)
+        return _exchage_code_for_token(console, options, token_code, description=desc)
 
 
-def _exchage_code_for_token(console: Console, options: AccessTokenAcquisitionOptions, token_code: str) -> AuthToken:
+def _exchage_code_for_token(
+    console: Console, options: AccessTokenAcquisitionOptions, token_code: str, description: str
+) -> AuthToken:
     # TODO: Use an engine intrinsic instead of directly going to the network.
     headers = get_common_request_headers()
-    data = {"code": token_code}
+    data = {"code": token_code, "desc": description}
     if options.ask_for_impersonation:
         data["allow_impersonation"] = "1"
     with requests.post(options.get_token_exchange_url(), data=data, headers=headers) as response:
         if not response.ok:
             console.print_stderr(console.red(_get_error_message(response)))
-            raise AuthError("Failed to acquire access token from server")
+            raise AuthError("Failed to acquire access token from server", context="auth-acquire")
         resp_data = response.json()
         return AuthToken.from_json_dict(resp_data)
 
@@ -235,7 +283,8 @@ def _acquire_token_headless(console: Console, options: AccessTokenAcquisitionOpt
     console.print_stdout(f"Using a web browser navigate to: {url}")
     # TODO: use console to get input from the user. https://github.com/pantsbuild/pants/issues/11398
     token_code = input("Type or paste in the token exchange code: ")
-    return _exchage_code_for_token(console, options, token_code)
+    desc = _get_token_desc(options)
+    return _exchage_code_for_token(console, options, token_code, description=desc)
 
 
 def _is_browser_available() -> bool:
@@ -259,3 +308,14 @@ def _get_error_message(response) -> str:
 
 def get_auth_rules():
     return collect_rules()
+
+
+def _get_token_desc(options: AccessTokenAcquisitionOptions) -> str:
+    if options.description:
+        return options.description
+    default_desc = socket.gethostname()
+    if options.log_only:
+        default_desc += " [for CI]"
+    # TODO: use console to get input from the user. https://github.com/pantsbuild/pants/issues/11398
+    user_desc = input(f"Enter token description [{default_desc}]: ")
+    return user_desc or default_desc
