@@ -20,8 +20,8 @@ from pants.goal.run_tracker import RunTrackerOptionEncoder
 from pants.option.option_value_container import OptionValueContainer
 from requests.exceptions import RequestException
 
-from toolchain.pants.auth.client import AuthToken
 from toolchain.pants.auth.store import AuthStore
+from toolchain.pants.auth.token import AuthToken
 from toolchain.pants.buildsense.common import Artifacts, WorkUnits
 from toolchain.pants.common.network import get_common_request_headers
 
@@ -32,9 +32,11 @@ class BuildSenseClient:
     _IMPERSONATE_HEADER = "X-Toolchain-Impersonate"
 
     @classmethod
-    def from_options(cls, client_options: OptionValueContainer, auth: AuthStore, repo: str) -> BuildSenseClient:
+    def from_options(
+        cls, client_options: OptionValueContainer, auth: AuthStore, repo: str, base_url: str
+    ) -> BuildSenseClient:
         return cls(
-            base_url=client_options.base_url,
+            base_url=f"{base_url}/api/v1/repos/",
             auth=auth,
             repo=repo,
             timeout=client_options.timeout,
