@@ -6,6 +6,9 @@
 # The copyright notice above does not evidence any actual or intended publication of such source code.
 # Disclosure of this source code or any related proprietary information is strictly prohibited without
 # the express written permission of Toolchain Labs, Inc.
+
+from __future__ import annotations
+
 import base64
 import logging
 import os
@@ -18,7 +21,8 @@ from threading import Thread
 from typing import Dict, List, Mapping, Optional
 
 from pants.base.build_environment import get_git
-from pants.core.util_rules.pants_environment import PantsEnvironment
+from pants.engine.environment import CompleteEnvironment
+from pants.engine.internals import native_engine
 from pants.engine.rules import collect_rules, rule
 from pants.engine.streaming_workunit_handler import (
     StreamingWorkunitContext,
@@ -40,13 +44,6 @@ from toolchain.pants.common.toolchain_setup import ToolchainSetup
 logger = logging.getLogger(__name__)
 
 
-_CI_MAP = {
-    "CIRCLECI": re.compile(r"^CIRCLE.*"),
-    "TRAVIS": re.compile(r"^TRAVIS.*"),
-    "GITHUB_ACTIONS": re.compile(r"^GITHUB.*"),
-}
-
-
 def optional_dir_option(dn: str) -> str:
     # Similar to Pant's dir_option, but doesn't require the directory to exist.
     return os.path.normpath(dn)
@@ -58,13 +55,7 @@ class Reporter(Subsystem):
 
     @classmethod
     def register_options(cls, register):
-        register(
-            "--base-url",
-            advanced=True,
-            type=str,
-            default="https://app.toolchain.com/api/v1/repos/",
-            help="Where to make HTTP requests to",
-        )
+
         register(
             "--timeout",
             advanced=True,
@@ -104,9 +95,23 @@ class Reporter(Subsystem):
         register(
             "--enable",
             type=bool,
-            default=False,
+            default=True,
             help="Enables the BuildSense reporter plugin",
         )
+        register(
+            "--log-upload",
+            type=bool,
+            default=True,
+            advanced=True,
+            help="Upload pants logs to buildsense",
+        )
+        register(
+            "--ci-env-scrub-terms",
+            type=list,
+            default=list(CaptureCIEnv.DEFAULT_EXCLUDE_TERMS),
+            advanced=True,
+            help="patterns for environment variables to exclude from uploaded CI env variables",
+        )
 
 
 class ReporterCallback(WorkunitsCallback):
@@ -117,7 +122,8 @@ class ReporterCallback(WorkunitsCallback):
         options: OptionValueContainer,
         auth_store: AuthStore,
         env: Mapping[str, str],
-        repo_name: Optional[str],
+        repo_name: str | None,
+        base_url: str,
     ):
         super().__init__()
         self._env = env
@@ -128,7 +134,9 @@ class ReporterCallback(WorkunitsCallback):
         if not repo_name:
             logger.warning("Couldn't determine repo name. BuildSense plugin will be disabled.")
             return
-        client = BuildSenseClient.from_options(client_options=options, auth=auth_store, repo=repo_name)
+        client = BuildSenseClient.from_options(
+            client_options=options, auth=auth_store, repo=repo_name, base_url=base_url
+        )
         # set self._build_state *before* changing the state.
         self._build_state = BuildState(
             client,
@@ -138,11 +146,16 @@ class ReporterCallback(WorkunitsCallback):
         )
 
         self._enabled = True
-        self._ci_regex_pattern = re.compile(options.ci_env_var_pattern) if options.ci_env_var_pattern else None
+        self._log_upload = options.log_upload
         self._call_count = 0
         self._reporter_thread = ReportThread(self._build_state)
+        self._ci_capture = CaptureCIEnv(pattern=options.ci_env_var_pattern, exclude_terms=options.ci_env_scrub_terms)
         logger.debug("BuildSense Plugin enabled")
 
+    @property
+    def can_finish_async(self) -> bool:
+        return True
+
     def __call__(
         self,
         *,
@@ -196,7 +209,7 @@ class ReporterCallback(WorkunitsCallback):
         self._reporter_thread.stop_thread()
 
     def _get_run_tracker_info(self, context: StreamingWorkunitContext) -> RunTrackerBuildInfo:
-        ci_env = _capture_ci_env(self._env, self._ci_regex_pattern)  # type: ignore[arg-type]
+        ci_env = self._ci_capture.capture(self._env)
         run_tracker = context.run_tracker
         has_ended = run_tracker.has_ended()
         run_info = run_tracker.run_information()
@@ -225,9 +238,8 @@ class ReporterCallback(WorkunitsCallback):
             observation_histograms = _get_historgrams(context)
             if observation_histograms:
                 build_stats["observation_histograms"] = observation_histograms
-
-        log_file = Path(run_tracker.run_logs_file) if has_ended else None
-
+        upload_log = all((has_ended, self._log_upload, run_tracker.run_logs_file))
+        log_file = Path(run_tracker.run_logs_file) if upload_log else None
         return RunTrackerBuildInfo(has_ended=has_ended, build_stats=build_stats, log_file=log_file)
 
 
@@ -255,12 +267,15 @@ def _get_historgrams(context: StreamingWorkunitContext) -> Optional[dict]:
     }
 
 
-def _adjust_run_info_fields(run_info: dict, goals: List[str], has_ended: bool) -> None:
-    scm = get_git()
-    revision = scm.commit_id
+def _adjust_run_info_fields(run_info: dict, goals: list[str], has_ended: bool) -> None:
     host = socket.gethostname()
-    machine = f"{host} [docker]" if _is_docker() else host
-    run_info.update(machine=machine, revision=scm.commit_id, branch=scm.branch_name or revision)
+    run_info["machine"] = f"{host} [docker]" if _is_docker() else host
+    scm = get_git()
+    if scm:
+        revision = scm.commit_id
+        run_info.update(revision=revision, branch=scm.branch_name or revision)
+    else:
+        logger.warning("Can't get git scm info")
 
     if "parent_build_id" in run_info:
         del run_info["parent_build_id"]
@@ -279,6 +294,7 @@ def _is_docker() -> bool:
 
 class ReportThread:
     def __init__(self, build_state: BuildState) -> None:
+        self._logging_destination = native_engine.stdio_thread_get_destination()
         self._build_state = build_state
         self._terminate = False
         self._reporter_thread = Thread(target=self._report_loop, name="buildsense-reporter", daemon=True)
@@ -289,6 +305,7 @@ class ReportThread:
         self._reporter_thread.join()
 
     def _report_loop(self):
+        native_engine.stdio_thread_set_destination(self._logging_destination)
         while not self._terminate:
             operation = self._build_state.send_report()
             if operation.is_sent():
@@ -298,18 +315,49 @@ class ReportThread:
         self._build_state.send_final_report()
 
 
-def _get_pattern(env: Dict[str, str]) -> Optional[re.Pattern]:
-    for ci_name, capture_expression in _CI_MAP.items():
-        if ci_name in env:
-            return capture_expression
-    return None
+class CaptureCIEnv:
+    _CI_MAP = {
+        "CIRCLECI": re.compile(r"^CIRCLE.*"),
+        "TRAVIS": re.compile(r"^TRAVIS.*"),
+        "GITHUB_ACTIONS": re.compile(r"^GITHUB.*"),
+    }
+    DEFAULT_EXCLUDE_TERMS = ("ACCESS", "TOKEN", "SECRET")
 
+    def __init__(self, *, pattern: str | None, exclude_terms: list[str]) -> None:
+        self._pattern = re.compile(pattern) if pattern else None
+        self._exclude_expression = re.compile("|".join(f".*{re.escape(v)}.*" for v in exclude_terms))
 
-def _capture_ci_env(env: Dict[str, str], pattern: Optional[re.Pattern]) -> Optional[Dict[str, str]]:
-    pattern = pattern or _get_pattern(env)
-    if not pattern:
+    def _get_pattern(self, env: Mapping[str, str]) -> re.Pattern | None:
+        for ci_name, capture_expression in self._CI_MAP.items():
+            if ci_name in env:
+                return capture_expression
         return None
-    return {key: value for key, value in env.items() if pattern.match(key)}
+
+    def capture(self, env: Mapping[str, str]) -> dict[str, str] | None:
+        captured = self._capture_ci_env(env)
+        return self._scrub(captured) if captured else None
+
+    def _capture_ci_env(self, env: Mapping[str, str]) -> dict[str, str] | None:
+        pattern = self._pattern or self._get_pattern(env)
+        if not pattern:
+            return None
+        return {key: value for key, value in env.items() if pattern.match(key)}
+
+    def _scrub(self, captured: dict[str, str]) -> dict[str, str]:
+        scrubbed_keys = set()
+        final_data = {}
+        for key, value in captured.items():
+            if self._exclude_expression.match(key):
+                scrubbed_keys.add(key)
+            else:
+                final_data[key] = value
+        captured_str = ",".join(sorted(captured.keys()))
+        if scrubbed_keys:
+            final_captured_str = ",".join(sorted(final_data.keys()))
+            logger.debug(f"captured CI env: {captured_str} scrubbed: {scrubbed_keys} final: {final_captured_str}")
+        else:
+            logger.debug(f"captured CI env: {captured_str}")
+        return final_data
 
 
 class BuildsenseCallbackFactoryRequest:
@@ -322,15 +370,16 @@ def construct_buildsense_callback(
     reporter: Reporter,
     toolchain_setup: ToolchainSetup,
     auth_store: AuthStore,
-    pants_environment: PantsEnvironment,
+    environment: CompleteEnvironment,
 ) -> WorkunitsCallbackFactory:
     repo_name = toolchain_setup.safe_get_repo_name()
     return WorkunitsCallbackFactory(
         lambda: ReporterCallback(
             reporter.options,
             auth_store=auth_store,
-            env=dict(pants_environment.env),
+            env=dict(environment),
             repo_name=repo_name,
+            base_url=toolchain_setup.base_url,
         )
     )
 
