@@ -14,8 +14,6 @@ from enum import Enum, unique
 from pathlib import Path
 from typing import List, Optional, Tuple
 
-from pants.util.logging import LogLevel
-
 from toolchain.base.datetime_tools import utcnow
 from toolchain.pants.buildsense.client import BuildSenseClient
 from toolchain.pants.buildsense.common import Artifacts, RunTrackerBuildInfo, WorkUnitsMap
@@ -120,17 +118,11 @@ class BuildState:
 
     def _get_log_file(self, run_tracker_info: RunTrackerBuildInfo) -> Optional[bytes]:
         if not run_tracker_info.log_file:
-            logger.debug("no log file associated in run tracker")
+            logger.warning("no log file associated in run tracker")
             return None
-        build_stats = run_tracker_info.build_stats
-        is_fail = build_stats["run_info"]["outcome"] != "SUCCESS"
-        log_level = build_stats["recorded_options"]["GLOBAL"].get("level", "")
-        if isinstance(log_level, LogLevel):
-            log_level = log_level.value.lower()
-        logger.debug(f"get_log_file log_level={log_level} is_fail={is_fail}")
-        if not is_fail and log_level not in {"debug", "trace"}:
-            return None
-        return run_tracker_info.log_file.read_bytes()
+        log_data = run_tracker_info.log_file.read_bytes()
+        logger.debug(f"get_log_file log_file={run_tracker_info.log_file.as_posix()} size={len(log_data)}")
+        return log_data
 
     def _get_artifacts(self, run_tracker_info: RunTrackerBuildInfo) -> Artifacts:
         artifacts = self._converter.get_standalone_artifacts() or {}
@@ -189,7 +181,7 @@ class BuildState:
         call_num, workunits, timestamp = self._get_lastest_workunits()
         if not workunits:
             return False
-        logger.debug(f"send_workunit_queue {len(workunits)} {call_num}")
+        logger.debug(f"send_workunit_queue workunits={len(workunits)} call={call_num}")
         run_id = self._run_id
         if not run_id:
             raise InvalidStateError("run_id must be initialized.")
