@@ -7,7 +7,7 @@
 # Disclosure of this source code or any related proprietary information is strictly prohibited without
 # the express written permission of Toolchain Labs, Inc.
 
-from typing import Optional
+from __future__ import annotations
 
 from pants.engine.rules import SubsystemRule
 from pants.option.subsystem import Subsystem
@@ -24,10 +24,21 @@ class ToolchainSetup(Subsystem):
     @classmethod
     def register_options(cls, register) -> None:
         register("--repo", type=str, help="Repo name", default=None)
+        register(
+            "--base-url",
+            type=str,
+            default="https://app.toolchain.com",
+            advanced=True,
+            help="Toolchain base url",
+        )
 
-    def safe_get_repo_name(self) -> Optional[str]:
+    def safe_get_repo_name(self) -> str | None:
         return self.options.repo or None
 
+    @property
+    def base_url(self) -> str:
+        return self.options.base_url
+
     def get_repo_name(self) -> str:
         repo = self.safe_get_repo_name()
         if not repo:
